#ifndef SERVER_H
#define SERVER_H

/* 
 * This header defines the socks proxy wrapper class
 */

#include <boost/shared_ptr.hpp>
#include <boost/asio.hpp>
#include <boost/thread.hpp>
#include <map>

#include "Packet.h"

#include "Packetizer.h"
#include "Client.h" 
#include "Morpher.h"
#include "Crypto.h"

using namespace boost::asio;


class Server {
private:
	boost::asio::io_service              _io_service;
	std::map<ip::udp::endpoint,
	boost::shared_ptr<Client> > _clients;
	boost::shared_ptr<ip::udp::socket>   _udpsocket;
	ip::udp::endpoint                    _sender_ep;
	uint16_t                             _rport;
	Morpher *                            _morpher;
	boost::thread                        _thread;
	unsigned char                        _buf[MAX_UDP_MSG_LEN];

	void serverSession();

	void recvUDPData();
	void handleUDPData(const boost::system::error_code &err,
			std::size_t bytes_transferred);

	bool verifyConnectionRequest(boost::shared_ptr<Client> & c,
			boost::shared_ptr<Packetizer> & pkzer);


	void clientSession(boost::shared_ptr<Client> & c, boost::shared_ptr<Codec> & codec);

public:

	Server(int listenPort,
			uint16_t remotePort,
			Morpher * morpher);

	boost::thread & run();

	void stop();

	void newClientSession(const std::string & addr, uint16_t port, boost::shared_ptr<Codec> & codec);
};




#endif

