#include "SkypeServer.h"
#include "Crypto.h"
#include "Util.h"
#include "debug.h"

#include <cstring>
#include <cassert>
#include <iostream>
#include <string>
#include <boost/function.hpp>
#include <boost/bind.hpp>



void SkypeServer::serverSession() {
  while (running) {

    std::cerr << "[skype] " << "Creating skype" << std::endl; 

    // exec skrt

    boost::shared_ptr<Codec> codec(new Codec(true)); 

    std::string clientAddr; 
    uint16_t    clientPort; 
    skype = new SMServerSkype(this, codec, boost::bind(&SkypeServer::handleClientInfo, 
                                                       this, codec, _1, _2));

    std::cerr << "[skype] " << "Submitting application token" << std::endl;

    readKeyPair(); 
    skype->init(keyBuf, skrtAddr, skrtPort, "skypesrv.log");
    skype->start();

    std::cerr << "[skype] " << "Retrieving account" << std::endl;

    SMAccount::Ref account;
    if(skypeLogin(account))
    {
      SMServerSkype * sk = dynamic_cast<SMServerSkype *>(skype); 
      assert(sk);
      skype->GetConversationList(sk->inbox, Conversation::INBOX_CONVERSATIONS);
      fetch(sk->inbox);
      {
        boost::unique_lock<boost::mutex> lock(mx);

        while (!done) cv.wait(lock);
      }
      skypeLogout(account);      
    }
    else
    {
      std::cerr << "[skype] " << "Login failed" << std::endl;
    }
    skype->stop(); 
    delete skype; running = false; 
  }
}

boost::thread & SkypeServer::run() {
  running = true; 

  thread = boost::thread(&SkypeServer::serverSession, this); 
  return thread; 
}

void SkypeServer::stop() {
  running = false; 
  thread.interrupt(); 
  thread.join();
}

void SkypeServer::handleClientInfo(boost::shared_ptr<Codec> codec,  
                                   std::string addr, uint16_t port) {
  boost::unique_lock<boost::mutex> lock(mx);
  server->newClientSession(addr, port, codec);

  done = true; 
  cv.notify_all();
}
